/*
 * Copyright (c) 2016, Freescale Semiconductor, Inc.
 * Copyright 2016-2017 NXP
 *
 * Redistribution and use in source and binary forms, with or without modification,
 * are permitted provided that the following conditions are met:
 *
 * o Redistributions of source code must retain the above copyright notice, this list
 *   of conditions and the following disclaimer.
 *
 * o Redistributions in binary form must reproduce the above copyright notice, this
 *   list of conditions and the following disclaimer in the documentation and/or
 *   other materials provided with the distribution.
 *
 * o Neither the name of the copyright holder nor the names of its
 *   contributors may be used to endorse or promote products derived from this
 *   software without specific prior written permission.
 *
 * THIS SOFTWARE IS PROVIDED BY THE COPYRIGHT HOLDERS AND CONTRIBUTORS "AS IS" AND
 * ANY EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT LIMITED TO, THE IMPLIED
 * WARRANTIES OF MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE ARE
 * DISCLAIMED. IN NO EVENT SHALL THE COPYRIGHT HOLDER OR CONTRIBUTORS BE LIABLE FOR
 * ANY DIRECT, INDIRECT, INCIDENTAL, SPECIAL, EXEMPLARY, OR CONSEQUENTIAL DAMAGES
 * (INCLUDING, BUT NOT LIMITED TO, PROCUREMENT OF SUBSTITUTE GOODS OR SERVICES;
 * LOSS OF USE, DATA, OR PROFITS; OR BUSINESS INTERRUPTION) HOWEVER CAUSED AND ON
 * ANY THEORY OF LIABILITY, WHETHER IN CONTRACT, STRICT LIABILITY, OR TORT
 * (INCLUDING NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY OUT OF THE USE OF THIS
 * SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF SUCH DAMAGE.
 */

/*
 * TEXT BELOW IS USED AS SETTING FOR THE PINS TOOL *****************************
PinsProfile:
- !!product 'Pins v2.0'
- !!processor 'LPC54608J512'
- !!package 'LPC54608J512ET180'
- !!mcu_data 'ksdk2_0'
- !!processor_version '0.0.0'
 * BE CAREFUL MODIFYING THIS COMMENT - IT IS YAML SETTINGS FOR THE PINS TOOL ***
 */

#include "fsl_common.h"
#include "fsl_iocon.h"
#include "fsl_gpio.h"
#include "pin_mux.h"

#define IOCON_PIO_DIGITAL_EN        0x0100u   /*!< Enables digital function */
#define IOCON_PIO_FUNC0               0x00u   /*!< Selects pin function 0 */
#define IOCON_PIO_FUNC1               0x01u   /*!< Selects pin function 1 */
#define IOCON_PIO_FUNC6               0x06u   /*!< Selects pin function 6 */
#define IOCON_PIO_INPFILT_OFF       0x0200u   /*!< Input filter disabled */
#define IOCON_PIO_INV_DI              0x00u   /*!< Input function is not inverted */
#define IOCON_PIO_MODE_INACT          0x00u   /*!< No addition pin function */
#define IOCON_PIO_MODE_PULLUP         0x20u   /*!< Selects pull-up function */
#define IOCON_PIO_OPENDRAIN_DI        0x00u   /*!< Open drain is disabled */
#define IOCON_PIO_SLEW_STANDARD       0x00u   /*!< Standard mode, output slew rate control is enabled */
#define PIN1_IDX                         1u   /*!< Pin number for pin 1 in a port 1 */
#define PIN10_IDX                       10u   /*!< Pin number for pin 10 in a port 0 */
#define PIN29_IDX                       29u   /*!< Pin number for pin 29 in a port 0 */
#define PIN30_IDX                       30u   /*!< Pin number for pin 30 in a port 0 */
#define PORT0_IDX                        0u   /*!< Port index */
#define PORT1_IDX                        1u   /*!< Port index */

/*
 * TEXT BELOW IS USED AS SETTING FOR THE PINS TOOL *****************************
BOARD_InitPins:
- options: {coreID: singlecore, enableClock: 'true'}
- pin_list:
  - {pin_num: B13, peripheral: FLEXCOMM0, signal: RXD_SDA_MOSI, pin_signal: PIO0_29/FC0_RXD_SDA_MOSI/CTIMER2_MAT3/SCT0_OUT8/TRACEDATA(2), mode: inactive, invert: disabled,
    glitch_filter: disabled, slew_rate: standard, open_drain: disabled}
  - {pin_num: A2, peripheral: FLEXCOMM0, signal: TXD_SCL_MISO, pin_signal: PIO0_30/FC0_TXD_SCL_MISO/CTIMER0_MAT0/SCT0_OUT9/TRACEDATA(1), mode: inactive, invert: disabled,
    glitch_filter: disabled, slew_rate: standard, open_drain: disabled}
  - {pin_num: K12, peripheral: GPIO, signal: 'PIO1, 1', pin_signal: PIO1_1/FC3_RXD_SDA_MOSI/CTIMER0_CAP3/SCT0_GPI5/USB1_OVERCURRENTN, mode: pullUp, invert: disabled,
    glitch_filter: disabled, slew_rate: standard, open_drain: disabled}
  - {pin_num: P2, peripheral: SWD, signal: SWO, pin_signal: PIO0_10/FC6_SCK/CTIMER2_CAP2/CTIMER2_MAT0/FC1_TXD_SCL_MISO/SWO/ADC0_0, mode: inactive, invert: disabled,
    glitch_filter: disabled, open_drain: disabled}
 * BE CAREFUL MODIFYING THIS COMMENT - IT IS YAML SETTINGS FOR THE PINS TOOL ***
 */

/*FUNCTION**********************************************************************
 *
 * Function Name : BOARD_InitPins
 * Description   : Configures pin routing and optionally pin electrical features.
 *
 *END**************************************************************************/

void IOFunctionInit(void) {
  int i, j;
  CLOCK_EnableClock(kCLOCK_Iocon);                           /* Enables the clock for the IOCON block. 0 = Disable; 1 = Enable.: 0x01u */

  const uint32_t GpioFuncConfig = (
    IOCON_PIO_FUNC0 |                                        /* Pin is configured as PIO0_0 */
    IOCON_PIO_MODE_INACT |                                   /* No addition pin function */
    IOCON_PIO_DIGITAL_EN |                                   /* Enables digital function */
    IOCON_PIO_INPFILT_OFF                                  /* Input filter disabled */
  );

  for(j = 0; j < 5; j++) {
    for(i = 0; i < 32; i ++) {
      // avoid to disable SWDIO and SWCLK
//      if((j == 0) && ((i == 11) || (i == 12))) {
//    	  continue;
//      }
	  if(i == 4 && j > 16) {
		  break;
	  }
      IOCON_PinMuxSet(IOCON, j, i, GpioFuncConfig);
    }
  }

  CLOCK_DisableClock(kCLOCK_Iocon);
  return;
}

void GpioInit(void) {
  CLOCK_EnableClock(kCLOCK_Gpio0);
  CLOCK_EnableClock(kCLOCK_Gpio1);
  CLOCK_EnableClock(kCLOCK_Gpio2);
  CLOCK_EnableClock(kCLOCK_Gpio3);
  CLOCK_EnableClock(kCLOCK_Gpio4);
//  CLOCK_EnableClock(kCLOCK_Gpio5);

  gpio_pin_config_t ConfigOutLow =
  {
    kGPIO_DigitalOutput,
    0,
  };
  gpio_pin_config_t ConfigOutHigh =
  {
    kGPIO_DigitalOutput,
    1,
  };
  gpio_pin_config_t ConfigIn =
  {
    kGPIO_DigitalInput,
    0,
  };
  int i, j;
  for(i = 0; i < 5; i++) {
	  for(j = 0; j < 32; j++){
		  if(i == 4 && j > 16) {
			  break;
		  }
		  GPIO_PinInit(GPIO, i, j, &ConfigOutLow);
	  }
  }
  GPIO_PinInit(GPIO, 3, 4, &ConfigIn);
  GPIO_PinInit(GPIO, 4, 9, &ConfigIn);
  GPIO_PinInit(GPIO, 0, 11, &ConfigIn);
  GPIO_PinInit(GPIO, 0, 12, &ConfigIn);

  CLOCK_DisableClock(kCLOCK_Gpio0);
  CLOCK_DisableClock(kCLOCK_Gpio1);
  CLOCK_DisableClock(kCLOCK_Gpio2);
  CLOCK_DisableClock(kCLOCK_Gpio3);
  CLOCK_DisableClock(kCLOCK_Gpio4);
//  CLOCK_DisableClock(kCLOCK_Gpio5);

  return;
}

void LowConsumpPinConfig(void) {
	IOFunctionInit();
	GpioInit();
	return;
}

void TurnPinLevelConfig() {
	  CLOCK_EnableClock(kCLOCK_Gpio0);
	  CLOCK_EnableClock(kCLOCK_Gpio1);
	  CLOCK_EnableClock(kCLOCK_Gpio2);
	  CLOCK_EnableClock(kCLOCK_Gpio3);
	  CLOCK_EnableClock(kCLOCK_Gpio4);
//	  CLOCK_EnableClock(kCLOCK_Gpio5);
  gpio_pin_config_t ConfigOutLow =
  {
    kGPIO_DigitalOutput,
    0,
  };
  gpio_pin_config_t ConfigOutHigh =
  {
    kGPIO_DigitalOutput,
    1,
  };
  gpio_pin_config_t ConfigIn =
  {
    kGPIO_DigitalInput,
    0,
  };

  int i, j;
  for(i = 0; i < 5; i++) {
	  for(j = 0; j < 32; j++){
		  if(i == 1 && j == 1) {
			  continue;
          }
		  if(i == 4 && j > 16) {
			  break;
          }
      printf("PIO%d_%d\n", i, j);
      volatile unsigned int dir = GPIO->DIR[i];
      volatile unsigned int level = GPIO->B[i][j];
//       GPIO_TogglePinsOutput(GPIO, i, (1 << j));
     GPIO_PinInit(GPIO, i, j, &ConfigIn);
      __WFI();
      if((dir | (1 << j)) == 0) {
        GPIO_PinInit(GPIO, i, j, &ConfigIn);
      } else {
        if(level != 0) {
          GPIO_PinInit(GPIO, i, j, &ConfigOutHigh);
        } else {
          GPIO_PinInit(GPIO, i, j, &ConfigOutLow);
        }
      }
//       GPIO_TogglePinsOutput(GPIO, i, (1 << j));
	  }
  }
}

/*******************************************************************************
 * EOF
 ******************************************************************************/
